' This Script creates a new cusotm property on every item in the selected group.
' It also assigns a new parameter called 'BoneID' that identifies the bones
' index in the group. This group is used to manage all deformers on the character



sub ConfigureICEBoneParamsProperty( oGrp )
	if typename( oGrp ) <> "Group" then
		logmessage "You must select Group." 
		exit sub
	end if
	
	for i=0 to oGrp.Members.Count - 1

		set oObj = oGrp.Members( i )
		
		set oICEBoneProp = Dictionary.GetObject( oObj + ".ICEBoneParams", false )

		if typename(oICEBoneProp) = "Nothing" then
			set oICEBoneProp = oObj.AddCustomProperty("ICEBoneParams", false )
		end if
		
		set oParam = oICEBoneProp.Parameters( "BoneID" )
		if typename(oParam) = "Nothing" then
			set oParam = oICEBoneProp.AddParameter3( "BoneID", siInt2, -1, -1, 999 )
		end if
		oParam.PutValue2 0, i
	next

	for i=0 to oGrp.Members.Count - 1
		
		set oObj = oGrp.Members( i )
		set oICEBoneProp = Dictionary.GetObject( oObj + ".ICEBoneParams", false )
		set oParentParam = oICEBoneProp.Parameters( "ParentID" )
		if typename(oParentParam ) = "Nothing" then
			set oParentParam = oICEBoneProp.AddParameter3( "ParentID", siInt2, -1, -1, 999 )
		end if
		
		set oObjParent = oObj.Parent
		if oGrp.IsMember( oObjParent ) then
			oParentParam.PutValue2 0, GetValue( oObjParent + ".ICEBoneParams.BoneID" )
		else
			oParentParam.PutValue2 0, -1
		end if

	next

	lengthThreshold = 10.0
	for i=0 to oGrp.Members.Count - 1
			
		set oObj = oGrp.Members( i )
		set oICEBoneProp = Dictionary.GetObject( oObj + ".ICEBoneParams" )		

		' ////////////////////////////////
		' Bone Length
		boneLength = 0.0
		
		for j=0 to oObj.Children.Count - 1
			set oSIChildLocalTransformation = oObj.Children.Item(j).Kinematics.Local.Transform
			set oChildLocalVector = XSIMath.CreateVector3
			oSIChildLocalTransformation.GetTranslation oChildLocalVector
			
			childXOffset = oChildLocalVector.X
			childYZOffset = (( abs( oChildLocalVector.Y) + abs(oChildLocalVector.Z) ) / 2.0 )
			
			lengthOffsetRatio = 0.0
			if childYZOffset > 0.0 then
				lengthOffsetRatio = childXOffset / childYZOffset
			elseif oChildLocalVector.X > 0.0 then
				lengthOffsetRatio = 99.0
			end if
		
			if ( childXOffset > boneLength ) And ( lengthOffsetRatio > lengthThreshold ) then
				' LogMessage "\t" & i & " : " & oObj.Children.Item(j).Name & " childXOffset : " & childXOffset
				' LogMessage "\t" & i & " : " & oObj.Children.Item(j).Name & " lengthOffsetRatio : " & lengthOffsetRatio
		
				boneLength = oChildLocalVector.X
			end if
		next

		set oParam = oICEBoneProp.Parameters( "Length" )
		if typename(oParam ) = "Nothing" then
			set oParam = oICEBoneProp.AddParameter3( "Length", siDouble, 0.0, 0.0, 999.0 )
		end if
		oParam.PutValue2 0, boneLength
		' logmessage boneLength

	next

end sub

' Usage: Select the Deformers grou and run this script.
ConfigureICEBoneParamsProperty Selection.Item( 0 )
